/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.search.test;

import org.apache.solr.client.solrj.SolrQuery;
import org.apache.solr.client.solrj.SolrServerException;
import org.apache.solr.client.solrj.response.QueryResponse;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.perforce.search.test.util.ScanningIndexTestBase;

public class ScanNoFstatsTest extends ScanningIndexTestBase {

    @BeforeClass
    public void setUp() throws Exception {
	this.configFileLocation = "/scannofstats.config";
	super.setUp();
    }

    @Test
    public void testScanToken() throws Exception {
	super.testCheckScanToken();
    }

    @Test
    public void testGetIndexedDocCount() {
	try {
	    SolrQuery q = new SolrQuery();
	    q.setQuery("*:*");
	    QueryResponse r;
	    r = this.solrServer.query(q);
	    // make sure every file revision was indexed (45 files x 3 revs each
	    // minus 1)
	    Assert.assertEquals(r.getResults().getNumFound(),
		    EXPECTED_INDEX_REV_COUNT);
	} catch (SolrServerException e) {
	    e.printStackTrace();
	    Assert.fail("Unhandled server exception");
	}
    }

    @Test
    public void testGetIndexedDigestCount() {
	try {
	    SolrQuery q = new SolrQuery();
	    q.setQuery("digest:*");
	    QueryResponse r;
	    r = this.solrServer.query(q);
	    // no fstats, no digest
	    Assert.assertEquals(r.getResults().getNumFound(), 0);
	} catch (SolrServerException e) {
	    e.printStackTrace();
	    Assert.fail("Unhandled server exception");
	}
    }

    @Test
    public void testGetSpecificFstatCount() {
	try {
	    SolrQuery q = new SolrQuery();
	    q.setQuery("p4attr_testattrib:*");
	    QueryResponse r;
	    r = this.solrServer.query(q);
	    Assert.assertEquals(r.getResults().getNumFound(), 0);
	    // also check the propagated attribs
	    q.setQuery("p4attr_testpropattrib:*");
	    r = this.solrServer.query(q);
	    Assert.assertEquals(r.getResults().getNumFound(), 0);
	} catch (SolrServerException e) {
	    e.printStackTrace();
	    Assert.fail("Unhandled server exception");
	}
    }
}
